@@ -4,7 +4,7 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   copyright (C) 2003-2014                                               *
+ *   copyright (C) 2003-2020                                               *
  *   Umbrello UML Modeller Authors <umbrello-devel@kde.org>                *
  ***************************************************************************/
 
@@ -18,7 +18,7 @@ class ClassGeneralPage;
 class ClassifierListPage;
 class ClassOptionsPage;
 class ConstraintListPage;
-class KFontChooser;
+class EntityWidget;
 class PackageContentsPage;
 class ObjectWidget;
 class UMLDoc;
@@ -29,7 +29,7 @@ class UMLWidgetStylePage;
 /**
  * @author Paul Hensgen <phensgen@techie.com>
  * @version 1.0
- * Bugs and comments to umbrello-devel@kde.org or http://bugs.kde.org
+ * Bugs and comments to umbrello-devel@kde.org or https://bugs.kde.org
  */
 class ClassPropertiesDialog : public MultiPageDialogBase
 {
@@ -58,6 +58,7 @@ protected:
     void setupOperationsPage();
     void setupTemplatesPage();
     void setupEnumLiteralsPage();
+    void setupEntityDisplayPage(EntityWidget *widget);
     void setupEntityAttributesPage();
     void setupEntityConstraintsPage();
     void setupContentsPage();
@@ -65,7 +66,6 @@ protected:
     void setupInstancePages();
 
 private:
-    KFontChooser*        m_pChooser;
     ClassGeneralPage*    m_pGenPage;
     ClassifierListPage*  m_pAttPage;
     ClassifierListPage*  m_pOpsPage;
