@@ -5,7 +5,7 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  *   copyright (C) 2003 Brian Thomas  <brian.thomas@gsfc.nasa.gov>         *
- *   copyright (C) 2004-2014                                               *
+ *   copyright (C) 2004-2020                                               *
  *   Umbrello UML Modeller Authors <umbrello-devel@kde.org>                *
  ***************************************************************************/
 
@@ -97,7 +97,11 @@ protected:
 
     void loadFromDocument();
 
+#if QT_VERSION >= 0x050000
+    QMenu * createPopup();
+#else
     KMenu * createPopup();
+#endif
 
 private:
     QString            m_parentDocName;
