@@ -4,7 +4,7 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   copyright (C) 2004-2014                                               *
+ *   copyright (C) 2004-2020                                               *
  *   Umbrello UML Modeller Authors <umbrello-devel@kde.org>                *
  ***************************************************************************/
 
@@ -32,6 +32,7 @@ ToolBarStateArrow::ToolBarStateArrow(UMLScene *umlScene)
  */
 ToolBarStateArrow::~ToolBarStateArrow()
 {
+    cleanup();
 }
 
 /**
@@ -41,6 +42,14 @@ void ToolBarStateArrow::init()
 {
     ToolBarState::init();
 
+    cleanup();
+}
+
+/**
+ * Clean up anything before deletion.
+ */
+void ToolBarStateArrow::cleanup()
+{
     while (!m_selectionRect.isEmpty())
         delete m_selectionRect.takeFirst();
     m_selectionRect.clear();
@@ -124,11 +133,7 @@ void ToolBarStateArrow::mouseReleaseWidget()
  */
 void ToolBarStateArrow::mouseReleaseEmpty()
 {
-    if (m_selectionRect.count() == 4) {
-        while (!m_selectionRect.isEmpty())
-            delete m_selectionRect.takeFirst();
-        m_selectionRect.clear();
-    }
+    cleanup();
 }
 
 /**
@@ -225,4 +230,3 @@ void ToolBarStateArrow::setCurrentWidget(UMLWidget* widget)
     ToolBarState::setCurrentWidget(widget);
 }
 
-#include "toolbarstatearrow.moc"
