@@ -4,7 +4,7 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   copyright (C) 2002-2014                                               *
+ *   copyright (C) 2002-2020                                               *
  *   Umbrello UML Modeller Authors <umbrello-devel@kde.org>                *
  ***************************************************************************/
 
@@ -30,6 +30,7 @@
 #include "interfacewidget.h"
 #include "notewidget.h"
 #include "messagewidget.h"
+#include "objectwidget.h"
 #include "object_factory.h"
 #include "idchangelog.h"
 #include "menus/listpopupmenu.h"
@@ -51,6 +52,7 @@
 #include "usecasewidget.h"
 #include "uniqueid.h"
 #include "widget_factory.h"
+#include "widget_utils.h"
 
 // kde includes
 #include <KLocalizedString>
@@ -66,6 +68,8 @@ using namespace Uml;
 
 DEBUG_REGISTER_DISABLED(UMLWidget)
 
+#define I18N_NEXT_RELEASE(a,b) QString(QLatin1String(a)).arg(b))
+
 const QSizeF UMLWidget::DefaultMinimumSize(50, 20);
 const QSizeF UMLWidget::DefaultMaximumSize(1000, 5000);
 const int UMLWidget::defaultMargin = 5;
@@ -80,16 +84,17 @@ const int UMLWidget::resizeMarkerLineCount = 3;
  * @param type  The WidgetType to construct.
  *              This must be set to the appropriate value by the constructors of inheriting classes.
  * @param o The UMLObject to represent.
+ * @note Although a pointer to the scene is required, the widget is not added to the scene by default.
  */
 UMLWidget::UMLWidget(UMLScene * scene, WidgetType type, UMLObject * o)
-  : WidgetBase(scene, type)
+  : WidgetBase(scene, type, o ? o->id() : Uml::ID::None)
   , DiagramProxyWidget(this)
 {
     init();
     m_umlObject = o;
     if (m_umlObject) {
+        // TODO: calling WidgetBase::setUMLObject does not add this connection
         connect(m_umlObject, SIGNAL(modified()), this, SLOT(updateWidget()));
-        m_nId = m_umlObject->id();
     }
 }
 
@@ -103,14 +108,10 @@ UMLWidget::UMLWidget(UMLScene * scene, WidgetType type, UMLObject * o)
  *  The default value (id_None) will prompt generation of a new ID.
  */
 UMLWidget::UMLWidget(UMLScene *scene, WidgetType type, Uml::ID::Type id)
-  : WidgetBase(scene, type)
+  : WidgetBase(scene, type, id)
   , DiagramProxyWidget(this)
 {
     init();
-    if (id == Uml::ID::None)
-        m_nId = UniqueID::gen();
-    else
-        m_nId = id;
 }
 
 /**
@@ -210,49 +211,6 @@ bool UMLWidget::operator==(const UMLWidget& other) const
      */
 }
 
-/**
- * Sets the local id of the object.
- *
- * @param id   The local id of the object.
- */
-void UMLWidget::setLocalID(Uml::ID::Type id)
-{
-    m_nLocalID = id;
-}
-
-/**
- * Returns the local ID for this object.  This ID is used so that
- * many objects of the same @ref UMLObject instance can be on the
- * same diagram.
- *
- * @return  The local ID.
- */
-Uml::ID::Type UMLWidget::localID() const
-{
-    return m_nLocalID;
-}
-
-/**
- * Returns the widget with the given ID.
- * The default implementation tests the following IDs:
- * - m_nLocalID
- * - if m_umlObject is non NULL: m_umlObject->id()
- * - m_nID
- * Composite widgets override this function to test further owned widgets.
- *
- * @param id  The ID to test this widget against.
- * @return  'this' if id is either of m_nLocalID, m_umlObject->id(), or m_nId;
- *           else NULL.
- */
-UMLWidget* UMLWidget::widgetWithID(Uml::ID::Type id)
-{
-    if (id == m_nLocalID ||
-        (m_umlObject != 0 && id == m_umlObject->id()) ||
-        id == m_nId)
-        return this;
-    return 0;
-}
-
 /**
  * Compute the minimum possible width and height.
  *
@@ -739,8 +697,6 @@ void UMLWidget::constrain(qreal& width, qreal& height)
  */
 void UMLWidget::init()
 {
-    m_nId = Uml::ID::None;
-    m_nLocalID = UniqueID::gen();
     m_isInstance = false;
     setMinimumSize(DefaultMinimumSize);
     setMaximumSize(DefaultMaximumSize);
@@ -875,6 +831,20 @@ void UMLWidget::slotMenuSelection(QAction *trigger)
         break;
     }
 
+    case ListPopupMenu::mt_Hide_Destruction_Box: {
+        ObjectWidget *w = asObjectWidget();
+        if (w)
+            w->setShowDestruction(false);
+        break;
+    }
+
+    case ListPopupMenu::mt_Show_Destruction_Box: {
+        ObjectWidget *w = asObjectWidget();
+        if (w)
+            w->setShowDestruction(true);
+        break;
+    }
+
     case ListPopupMenu::mt_Interface: {
         UMLPackage* component = umlObject()->asUMLPackage();
         QString name = Model_Utils::uniqObjectName(UMLObject::ot_Interface, component);
@@ -933,6 +903,8 @@ void UMLWidget::slotMenuSelection(QAction *trigger)
         break;
     }
 
+    case ListPopupMenu::mt_MessageCreation:
+    case ListPopupMenu::mt_MessageDestroy:
     case ListPopupMenu::mt_MessageSynchronous:
 //        MessageWidget *widget = new MessageWidget(umlScene(), this);
 //        addConnectedWidget(widget, Uml::AssociationType::Coll_Message_Synchronous);
@@ -1011,7 +983,7 @@ void UMLWidget::slotMenuSelection(QAction *trigger)
         addConnectedWidget(new StateWidget(umlScene(), StateWidget::ShallowHistory), Uml::AssociationType::State, NoOption);
         break;
     case ListPopupMenu::mt_State:
-        addConnectedWidget(new StateWidget(umlScene(), StateWidget::Normal), Uml::AssociationType::State, NoOption);
+        addConnectedWidget(new StateWidget(umlScene(), StateWidget::Normal), Uml::AssociationType::State, ShowProperties);
         break;
     case ListPopupMenu::mt_StateFork:
         addConnectedWidget(new StateWidget(umlScene(), StateWidget::Fork), Uml::AssociationType::State, NoOption);
@@ -1212,21 +1184,17 @@ void UMLWidget::setFillColorCmd(const QColor &color)
 }
 
 /**
- * Activate the object after serializing it from a QDataStream
+ * Reimplemented from class WidgetBase
  *
  * @param ChangeLog
  * @return  true for success
  */
 bool UMLWidget::activate(IDChangeLog* changeLog)
 {
+    if (!WidgetBase::activate(changeLog))
+        return false;
     DiagramProxyWidget::activate(changeLog);
-    if (widgetHasUMLObject(baseType()) && m_umlObject == 0) {
-        m_umlObject = m_doc->findObjectById(m_nId);
-        if (m_umlObject == 0) {
-            uError() << "cannot find UMLObject with id=" << Uml::ID::toString(m_nId);
-            return false;
-        }
-    }
+
     setFontCmd(m_font);
     setSize(width(), height());
     m_activated = true;
@@ -1301,8 +1269,7 @@ void UMLWidget::setActivated(bool active /*=true*/)
 }
 
 /**
- * Adds an already created association to the list of
- * associations that include this UMLWidget
+ * Reimplemented from class WidgetBase
  */
 void UMLWidget::addAssoc(AssociationWidget* pAssoc)
 {
@@ -1321,8 +1288,7 @@ AssociationWidgetList &UMLWidget::associationWidgetList() const
 }
 
 /**
- * Removes an already created association from the list of
- * associations that include this UMLWidget
+ * Reimplemented from class WidgetBase
  */
 void UMLWidget::removeAssoc(AssociationWidget* pAssoc)
 {
@@ -1718,7 +1684,7 @@ void UMLWidget::setScene(UMLScene *scene)
  */
 void UMLWidget::setX(qreal x)
 {
-    QGraphicsObject::setX(x);
+    QGraphicsObjectWrapper::setX(x);
 }
 
 /**
@@ -1730,7 +1696,7 @@ void UMLWidget::setX(qreal x)
  */
 void UMLWidget::setY(qreal y)
 {
-    QGraphicsObject::setY(y);
+    QGraphicsObjectWrapper::setY(y);
 }
 
 /**
@@ -1755,29 +1721,6 @@ void UMLWidget::slotSnapToGrid()
     }
 }
 
-/**
- * Returns whether the widget type has an associated UMLObject
- */
-bool UMLWidget::widgetHasUMLObject(WidgetBase::WidgetType type)
-{
-    if (type == WidgetBase::wt_Actor         ||
-            type == WidgetBase::wt_UseCase   ||
-            type == WidgetBase::wt_Class     ||
-            type == WidgetBase::wt_Interface ||
-            type == WidgetBase::wt_Enum      ||
-            type == WidgetBase::wt_Datatype  ||
-            type == WidgetBase::wt_Package   ||
-            type == WidgetBase::wt_Component ||
-            type == WidgetBase::wt_Port ||
-            type == WidgetBase::wt_Node      ||
-            type == WidgetBase::wt_Artifact  ||
-            type == WidgetBase::wt_Object) {
-        return true;
-    } else {
-        return false;
-    }
-}
-
 /**
  * Set m_ignoreSnapToGrid.
  */
@@ -1952,7 +1895,9 @@ void UMLWidget::paint(QPainter *painter, const QStyleOptionGraphicsItem *option,
         }
         // debug info
         if (Tracer::instance()->isEnabled(QLatin1String(metaObject()->className()))) {
-            painter->setPen(Qt::green);
+            QPen p(Qt::green);
+            p.setWidthF(1.0);
+            painter->setPen(p);
             painter->setBrush(Qt::NoBrush);
             painter->drawPath(shape());
             painter->setPen(Qt::blue);
@@ -2105,7 +2050,6 @@ void UMLWidget::saveToXMI1(QDomDocument & qDoc, QDomElement & qElement)
     */
     WidgetBase::saveToXMI1(qDoc, qElement);
     DiagramProxyWidget::saveToXMI1(qDoc, qElement);
-    qElement.setAttribute(QLatin1String("xmi.id"), Uml::ID::toString(id()));
 
     qreal dpiScale = UMLApp::app()->document()->dpiScale();
     qElement.setAttribute(QLatin1String("x"), QString::number(x() / dpiScale));
@@ -2118,17 +2062,10 @@ void UMLWidget::saveToXMI1(QDomDocument & qDoc, QDomElement & qElement)
         qElement.setAttribute(QLatin1String("instancename"), m_instanceName);
     if (m_showStereotype)
         qElement.setAttribute(QLatin1String("showstereotype"), m_showStereotype);
-
-    // Unique identifier for widget (todo: id() should be unique, new attribute
-    // should indicate the UMLObject's ID it belongs to)
-    qElement.setAttribute(QLatin1String("localid"), Uml::ID::toString(m_nLocalID));
 }
 
 bool UMLWidget::loadFromXMI1(QDomElement & qElement)
 {
-    QString id = qElement.attribute(QLatin1String("xmi.id"), QLatin1String("-1"));
-    m_nId = Uml::ID::fromString(id);
-
     WidgetBase::loadFromXMI1(qElement);
     DiagramProxyWidget::loadFromXMI1(qElement);
     QString x = qElement.attribute(QLatin1String("x"), QLatin1String("0"));
@@ -2147,11 +2084,6 @@ bool UMLWidget::loadFromXMI1(QDomElement & qElement)
     QString showstereo = qElement.attribute(QLatin1String("showstereotype"), QLatin1String("0"));
     m_showStereotype = (bool)showstereo.toInt();
 
-    QString localid = qElement.attribute(QLatin1String("localid"), QLatin1String("0"));
-    if (localid != QLatin1String("0")) {
-        m_nLocalID = Uml::ID::fromString(localid);
-    }
-
     return true;
 }
 
@@ -2163,6 +2095,15 @@ bool UMLWidget::loadFromXMI1(QDomElement & qElement)
  */
 void UMLWidget::addConnectedWidget(UMLWidget *widget, Uml::AssociationType::Enum type, AddWidgetOptions options)
 {
+    QString name = Widget_Utils::defaultWidgetName(widget->baseType());
+    widget->setName(name);
+    if (options & ShowProperties) {
+        if (!widget->showPropertiesDialog()) {
+            delete widget;
+            return;
+        }
+    }
+
     umlScene()->addItem(widget);
     widget->setX(x() + rect().width() + 100);
     widget->setY(y());
@@ -2176,8 +2117,12 @@ void UMLWidget::addConnectedWidget(UMLWidget *widget, Uml::AssociationType::Enum
     umlScene()->addAssociation(assoc);
     umlScene()->clearSelected();
     umlScene()->selectWidget(widget);
-    if (options & ShowProperties)
-        widget->showPropertiesDialog();
+
+    UMLApp::app()->beginMacro(I18N_NEXT_RELEASE("Adding connected '%1'", widget->baseTypeStrWithoutPrefix());
+    UMLApp::app()->executeCommand(new CmdCreateWidget(widget));
+    UMLApp::app()->executeCommand(new CmdCreateWidget(assoc));
+    UMLApp::app()->endMacro();
+    m_doc->setModified();
 }
 
 /**
